@@ -26,15 +26,30 @@
 
 #include <libraw1394/raw1394.h>
 #include <pthread.h>
+#include <getopt.h>
 
 #include "ieee1394io.h"
 #include "filehandler.h"
-#include "frame.h"
+#include "dvframe.h"
+#include "hdvframe.h"
 #include "smiltime.h"
 
 #include <stdint.h>
 
-#define RAW_BUF_SIZE    (10240)
+#define DEFAULT_FORMAT (AVI_DV2_FORMAT)
+#define DEFAULT_FORMAT_STR "raw"
+#define DEFAULT_FRAMES 0
+#define DEFAULT_LOCKSTEP_MAXDROPS -1
+#define DEFAULT_LOCKSTEP_TOTALDROPS -1
+#define DEFAULT_SIZE 1000
+#define DEFAULT_CSIZE 0
+#define DEFAULT_CMINCUTSIZE 0
+#define DEFAULT_EVERY 1
+#define DEFAULT_CHANNEL 63
+#define DEFAULT_BUFFERS 100
+#define DEFAULT_V4L2_DEVICE "/dev/video"
+
+extern int g_debug;
 
 class DVgrab
 {
@@ -43,7 +58,12 @@ private:
 	int	m_port;
 	int m_node;
 
+	char *m_program_name;
+
+	int m_showstatus;
+	bool m_hdv;
 	bool m_reader_active;
+	char *m_input_file_name;
 	char *m_dst_file_name;
 	int m_autosplit;
 	int m_timestamp;
@@ -54,32 +74,37 @@ private:
 	int m_collection_min_cut_file_size;
 	int m_sizesplitmode;
 	int m_file_format;
-	bool m_open_dml;
+	int m_open_dml;
 	int m_frame_every;
 	int m_jpeg_quality;
-	bool m_jpeg_deinterlace;
+	int m_jpeg_deinterlace;
 	int m_jpeg_width;
 	int m_jpeg_height;
-	bool m_jpeg_overwrite;
+	std::string m_jpeg_temp;
+	int m_jpeg_usetemp;
+	int m_jpeg_overwrite;
 	int m_dropped_frames;
 	bool m_interactive;
 	int m_buffers;
 	int m_total_frames;
 	std::string m_duration;
 	SMIL::MediaClippingTime* m_timeDuration;
-	bool m_stdin;
-	bool m_noavc;
+	int m_noavc;
 	uint64_t m_guid;
-	bool m_timesys;
+	int m_timesys;
 	iec61883Connection* m_connection;
 	bool m_raw_pipe;
-	bool m_no_stop;
-	bool m_timecode;
-	bool m_lockstep;
+	int m_no_stop;
+	int m_timecode;
+	int m_lockstep;
 	int m_lockstep_maxdrops;
 	int m_lockstep_totaldrops;
 	bool m_lockPending;
-	TimeCode m_prevTimeCode;
+	TimeCode m_lastTimeCode;
+	bool m_isLastTimeCodeSet;
+	struct tm m_lastRecDate;
+	bool m_isLastRecDateSet;
+	bool m_v4l2;
 
 	static FileHandler *m_writer;
 	bool m_captureActive;
@@ -108,7 +133,6 @@ public:
 	void watchdogThread();
 	void captureThreadRun();
 	bool execute( const char );
-	void sendEvent( const char *format, ... );
 	bool isPlaying();
 	bool isInteractive()
 	{
@@ -119,9 +143,17 @@ public:
 	static void testCaptureProxy( BusResetHandlerData );
 
 private:
-	void sendCaptureStatus( float size, TimeCode& timeCode, struct tm& recDate );
-	void writeFrame( TimeCode& timeCode, struct tm& recDate );
+	void sendCaptureStatus( const char *name, float size, int frames, TimeCode *tc, struct tm *rd, bool newline );
+	void sendFrameDroppedStatus( const char *reason, const char *meaning );
+	void writeFrame();
 	void cleanup();
+
+	void print_usage();
+	void print_help();
+	void print_version();
+	void set_file_format( char *format );
+	void set_format_from_name( void );
 };
 
+
 #endif
