@@ -42,6 +42,65 @@ using std::ends;
 
 #include "error.h"
 
+static bool needNewLine = false;
+
+#define MAX_DEBUG_PIDS 512
+static int pids[MAX_DEBUG_PIDS];
+static int n_pids = 0;
+
+bool d_all;
+bool d_hdv_pat;
+bool d_hdv_pmt;
+bool d_hdv_pids;
+bool d_hdv_pes;
+bool d_hdv_packet;
+bool d_hdv_video;
+bool d_hdv_sonya1;
+
+void d_hdv_pid_add( int pid )
+{
+	if ( n_pids < MAX_DEBUG_PIDS )
+		pids[n_pids++] = pid;
+}
+
+bool d_hdv_pid_check( int pid )
+{
+	for ( int i=0; i<n_pids; i++ )
+		if ( pids[i] == pid )
+			return true;
+	return false;
+}
+
+void sendEventParams( int preline, int postline, const char *format, ... )
+{
+	va_list list;
+	va_start( list, format );
+	char line[ 1024 ];
+	int ret;
+
+	if ( needNewLine )
+	{
+		if ( preline == 1 )
+			fprintf( stderr, "\n" );
+		else if ( preline == 2 )
+			fprintf( stderr, "\r%-80.80s\r", " " );
+	}
+
+	if ( postline )
+		ret = snprintf( line, 1024, "%s\n", format );
+	else
+		ret = snprintf( line, 1024, "%s", format );
+
+	if ( ret > 0 )
+		vfprintf( stderr, line, list );
+	va_end( list );
+
+	if ( postline )
+		needNewLine = false;
+	else
+		needNewLine = true;
+}
+
 void real_fail_neg( int eval, const char *eval_str, const char *func, const char *file, int line )
 {
 	if ( eval < 0 )
