@@ -28,7 +28,6 @@
 #include <sstream>
 #include <iomanip>
 
-using std::cerr;
 using std::cout;
 using std::endl;
 using std::ostringstream;
@@ -48,20 +47,22 @@ using std::setfill;
 #include "riff.h"
 #include "avi.h"
 #include "frame.h"
+#include "dvframe.h"
 #include "affine.h"
+#include "stringutils.h"
 
 FileTracker *FileTracker::instance = NULL;
 
 FileTracker::FileTracker( ) : mode( CAPTURE_MOVIE_APPEND )
 {
 	return ;
-	cerr << ">> Constructing File Capture tracker" << endl;
+	sendEvent( ">> Constructing File Capture tracker" );
 }
 
 FileTracker::~FileTracker( )
 {
 	return ;
-	cerr << ">> Destroying File Capture tracker" << endl;
+	sendEvent( ">> Destroying File Capture tracker" );
 }
 
 FileTracker &FileTracker::GetInstance( )
@@ -92,7 +93,7 @@ void FileTracker::Add( const char *file )
 	return ;
 	if ( this->mode != CAPTURE_IGNORE )
 	{
-		cerr << ">>>> Registering " << file << " with the tracker" << endl;
+		sendEvent( ">>>> Registering %s with the tracker", file );
 		list.push_back( strdup( file ) );
 	}
 }
@@ -112,23 +113,21 @@ void FileTracker::Clear( )
 	this->mode = CAPTURE_MOVIE_APPEND;
 }
 
-FileHandler::FileHandler() : done( false ), autoSplit( false ), maxFrameCount( 999999 ),
-		lastCollectionFreeSpace( 0 ), currentCollectionSize( 0 ), framesWritten( 0 ), filename( "" )
+FileHandler::FileHandler() :
+	done( false ), autoSplit( false ), maxFrameCount( 0 ), isNewFile( false ), isFirstFile( -1 ),
+	lastCollectionFreeSpace( 0 ), currentCollectionSize( 0 ), framesWritten( 0 ), filename( "" )
 {
-	/* empty body */
+	prevTimeCode.sec = -1;
 }
 
 
 FileHandler::~FileHandler()
 {
-	/* empty body */
 }
 
 
 void FileHandler::CollectionCounterUpdate()
 {
-	// Called when a split occurs!
-	//cerr << ">> Updating Collection Counter" << endl;
 	if ( GetSizeSplitMode() == 1) 
 	{
 		currentCollectionSize += GetFileSize();
@@ -155,63 +154,63 @@ void FileHandler::CollectionCounterUpdate()
 }
 
 
-bool FileHandler::GetAutoSplit() const
+bool FileHandler::GetAutoSplit()
 {
 	return autoSplit;
 }
 
 
-bool FileHandler::GetTimeStamp() const
+bool FileHandler::GetTimeStamp()
 {
 	return timeStamp;
 }
 
 
-bool FileHandler::GetTimeSys() const
+bool FileHandler::GetTimeSys()
 {
 	return timeSys;
 }
 
 
-bool FileHandler::GetTimeCode() const
+bool FileHandler::GetTimeCode()
 {
 	return timeCode;
 }
 
 
-string FileHandler::GetBaseName() const
+string FileHandler::GetBaseName()
 {
 	return base;
 }
 
 
-string FileHandler::GetExtension() const
+string FileHandler::GetExtension()
 {
 	return extension;
 }
 
 
-int FileHandler::GetMaxFrameCount() const
+int FileHandler::GetMaxFrameCount()
 {
 	return maxFrameCount;
 }
 
-off_t FileHandler::GetMaxFileSize() const
+off_t FileHandler::GetMaxFileSize()
 {
 	return maxFileSize;
 }
 
-int FileHandler::GetSizeSplitMode() const
+int FileHandler::GetSizeSplitMode()
 {
 	return sizeSplitMode;
 }
 
-off_t FileHandler::GetMaxColSize() const
+off_t FileHandler::GetMaxColSize()
 {
 	return maxColSize;
 }
 
-off_t FileHandler::GetMinColSize() const
+off_t FileHandler::GetMinColSize()
 {
 	return minColSize;
 }
@@ -284,11 +283,6 @@ void FileHandler::SetMinColSize( off_t size )
 	minColSize  = size;
 }
 
-void FileHandler::SetSampleFrame( const Frame& sample )
-{
-	/* empty body */
-}
-
 
 bool FileHandler::Done()
 {
@@ -296,31 +290,68 @@ bool FileHandler::Done()
 }
 
 
-bool FileHandler::WriteFrame( Frame& frame )
+bool FileHandler::WriteFrame( Frame *frame )
 {
-	/* If the user wants autosplit, start a new file if a
-	   new recording is detected. */
-	if ( FileIsOpen() && GetAutoSplit() && frame.IsNewRecording() )
+	/* If the file size, collection size, or max frame count would be exceeded
+	 * by this frame, and we can start a new file on this frame, close the current file.
+	 * If the autosplit flag is set, a new file will be created.
+	 */
+	if ( FileIsOpen() && frame->CanStartNewStream() )
+	{
+		bool startNewFile = false;
+		off_t newFileSize = GetFileSize() + frame->GetDataLen();
+		bool maxFileSizeExceeded = newFileSize >= GetMaxFileSize();
+		bool maxColSizeExceeded = GetCurrentCollectionSize() + newFileSize >= GetMaxColSize();
+
+		if ( GetFileSize() > 0 )
+		{
+			if ( GetMaxFileSize() > 0 && maxFileSizeExceeded )
+				startNewFile = true;
+
+			if ( GetMaxColSize() > 0 && GetSizeSplitMode() == 1 && maxColSizeExceeded )
+				startNewFile = true;
+		}
+
+		if ( GetMaxFrameCount() > 0 && framesWritten >= GetMaxFrameCount() )
+			startNewFile = true;
+
+		if ( startNewFile )
+		{
+			CollectionCounterUpdate();
+			Close();
+			done = !GetAutoSplit();
+		}
+	}
+
+	TimeCode tc;
+	int time_diff = 0;
+	if ( frame->GetTimeCode( tc ) )
+		time_diff = TIMECODE_TO_SEC(tc) - TIMECODE_TO_SEC(prevTimeCode);
+	bool discontinuity = prevTimeCode.sec != -1 && ( time_diff > 1 || ( time_diff < 0 && time_diff > -59 ) );
+
+	// If the user wants autosplit, start a new file if a new recording is detected
+	// either by explicit frame flag or a timecode discontinuity
+	if ( FileIsOpen() && GetAutoSplit() && ( frame->IsNewRecording() || discontinuity ) )
 	{
-		// Note: This doesn't really do anything, because the code 
-		//       at (dvgrab.cc:around m_writer->CollectionCounterUpdate)
-		//       actually preceeds this...
 		CollectionCounterUpdate();
 		Close();
 	}
 
+	isNewFile = false;
+
 	if ( ! FileIsOpen() )
 	{
-
 		static int counter = 0;
 
+		prevTimeCode.sec = -1;
+
 		if ( GetTimeStamp() )
 		{
 			ostringstream	sb, sb2;
 			struct tm	date;
 			string	recDate;
 
-			if ( frame.GetRecordingDate( date ) )
+			if ( frame->GetRecordingDate( date ) )
 			{
 				sb << setfill( '0' )
 				<< setw( 4 ) << date.tm_year + 1900 << '.'
@@ -337,7 +368,6 @@ bool FileHandler::WriteFrame( Frame& frame )
 			}
 			sb2 << GetBaseName() << recDate << GetExtension();
 			filename = sb2.str();
-			//cerr << ">>> Trying " << filename << endl;
 		}
 		else if ( GetTimeSys() )
 		{
@@ -365,7 +395,7 @@ TimeSys:
 			ostringstream sb;
 			TimeCode tc;
 			
-			if ( frame.GetTimeCode( tc ) )
+			if ( frame->GetTimeCode( tc ) )
 			{	
 				sb << GetBaseName()
 				<< setfill( '0' )
@@ -391,16 +421,20 @@ TimeSys:
 				ostringstream sb;
 				sb << GetBaseName() << setfill( '0' ) << setw( 3 ) << ++ counter << GetExtension();
 				filename = sb.str();
-				//cerr << ">>> Trying " << filename << endl;
 			}
 			while ( stat( filename.c_str(), &stats ) == 0 );
 		}
 
 		if ( ! Create( filename ) )
 		{
-			cerr << ">>> Error creating file!" << endl;
+			sendEvent( ">>> Error creating file!" );
 			return false;
 		}
+		isNewFile = true;
+		if ( isFirstFile == -1 )
+			isFirstFile = 1;
+		else if ( isFirstFile == 1 )
+			isFirstFile = 0;
 		framesWritten = 0;
 		framesToSkip = 0;
 	}
@@ -411,7 +445,7 @@ TimeSys:
 	{
 		if ( 0 > Write( frame ) )
 		{
-			cerr << ">>> Error writing frame!" << endl;
+			sendEvent( ">>> Error writing frame!" );
 			return false;
 		}
 		framesToSkip = everyNthFrame;
@@ -419,53 +453,12 @@ TimeSys:
 	}
 	framesToSkip--;
 
-	/* If the frame count is exceeded, close the current file.
-	   If the autosplit flag is set, a new file will be created in the next iteration.
-	   If the flag is not set, we are done. */
-
-	if ( ( GetMaxFrameCount() > 0 ) &&
-	        ( framesWritten >= GetMaxFrameCount() ) )
-	{
-		CollectionCounterUpdate();
-		Close();
-		done = !GetAutoSplit();
-	}
-
-	/* If the file size could be exceeded by another frame, close the current file.
-	   If the autosplit flag is set, a new file will be created on the next iteration.
-	   If the flag is not set, we are done. */
-	/* not exact, but should be good enough to prevent going over. */
-	if ( FileIsOpen() )
-	{
-		AudioInfo info;
-		frame.GetAudioInfo( info );
-		if
-		((
-			( GetFileSize() > 0 ) && ( GetMaxFileSize() > 0 ) &&
-			(
-				( GetFileSize() + frame.GetFrameSize() + info.samples * 4 + 12 )
-				   >= GetMaxFileSize() 
-			)
-		)
-		||
-		(
-			( GetFileSize() > 0 ) && ( GetMaxColSize() > 0 ) && ( GetSizeSplitMode() == 1 ) &&
-			(
-				( GetCurrentCollectionSize() + GetFileSize() + frame.GetFrameSize() + info.samples * 4 + 12 )
-				  >= GetMaxColSize()
-			)
-		))
-		{                     // 12 = sizeof chunk metadata
-			CollectionCounterUpdate();
-			Close();
-			done = !GetAutoSplit();
-		}
-	}
+	if ( !frame->GetTimeCode( prevTimeCode ) )
+		prevTimeCode.sec = -1;
 
 	return true;
 }
 
-
 /***************************************************************************/
 
 
@@ -502,9 +495,9 @@ bool RawHandler::Create( const string& filename )
 }
 
 
-int RawHandler::Write( Frame& frame )
+int RawHandler::Write( Frame *frame )
 {
-	int result = write( fd, frame.data, frame.GetFrameSize() );
+	int result = write( fd, frame->data, frame->GetDataLen() );
 	return result;
 }
 
@@ -557,7 +550,8 @@ bool RawHandler::Open( const char *s )
 
 }
 
-int RawHandler::GetFrame( Frame &frame, int frameNum )
+
+int RawHandler::GetFrame( Frame *frame, int frameNum )
 {
 	assert( fd != -1 );
 	int size = 480 * numBlocks;
@@ -565,11 +559,8 @@ int RawHandler::GetFrame( Frame &frame, int frameNum )
 		return -1;
 	off_t offset = ( ( off_t ) frameNum * ( off_t ) size );
 	fail_if( lseek( fd, offset, SEEK_SET ) == ( off_t ) - 1 );
-	if ( read( fd, frame.data, size ) > 0 )
-	{
-		frame.ExtractHeader();
+	if ( read( fd, frame->data, size ) > 0 )
 		return 0;
-	}
 	else
 		return -1;
 }
@@ -578,8 +569,8 @@ int RawHandler::GetFrame( Frame &frame, int frameNum )
 /***************************************************************************/
 
 
-AVIHandler::AVIHandler( int format ) : avi( NULL ), aviFormat( format ), isOpenDML( false ),
-		fccHandler( make_fourcc( "dvsd" ) )
+AVIHandler::AVIHandler( int format ) : avi( NULL ), filen( NULL ), aviFormat( format ), isOpenDML( false ),
+		fccHandler( make_fourcc( "dvsd" ) ), infoSet( false )
 {
 	extension = ".avi";
 }
@@ -591,34 +582,35 @@ AVIHandler::~AVIHandler()
 }
 
 
-void AVIHandler::SetSampleFrame( const Frame& sample )
+void AVIHandler::SetSampleFrame( DVFrame *sample )
 {
 	Pack pack;
 
-	sample.GetAudioInfo( audioInfo );
-	sample.GetVideoInfo( videoInfo );
+	sample->GetAudioInfo( audioInfo );
+	sample->GetVideoInfo( videoInfo );
 
-	sample.GetAAUXPack( 0x50, pack );
+	sample->GetAAUXPack( 0x50, pack );
 	dvinfo.dwDVAAuxSrc = *( DWORD* ) ( pack.data + 1 );
-	sample.GetAAUXPack( 0x51, pack );
+	sample->GetAAUXPack( 0x51, pack );
 	dvinfo.dwDVAAuxCtl = *( DWORD* ) ( pack.data + 1 );
 
-	sample.GetAAUXPack( 0x52, pack );
+	sample->GetAAUXPack( 0x52, pack );
 	dvinfo.dwDVAAuxSrc1 = *( DWORD* ) ( pack.data + 1 );
-	sample.GetAAUXPack( 0x53, pack );
+	sample->GetAAUXPack( 0x53, pack );
 	dvinfo.dwDVAAuxCtl1 = *( DWORD* ) ( pack.data + 1 );
 
-	sample.GetVAUXPack( 0x60, pack );
+	sample->GetVAUXPack( 0x60, pack );
 	dvinfo.dwDVVAuxSrc = *( DWORD* ) ( pack.data + 1 );
-	sample.GetVAUXPack( 0x61, pack );
+	sample->GetVAUXPack( 0x61, pack );
 	dvinfo.dwDVVAuxCtl = *( DWORD* ) ( pack.data + 1 );
 
 #ifdef WITH_LIBDV
 
-	if ( sample.decoder->std == e_dv_std_smpte_314m )
+	if ( sample->decoder->std == e_dv_std_smpte_314m )
 		fccHandler = make_fourcc( "dv25" );
 #endif
 
+	infoSet = true;
 }
 
 
@@ -631,6 +623,12 @@ bool AVIHandler::Create( const string& filename )
 {
 	assert( avi == NULL );
 
+	if ( !infoSet )
+	{
+		filen = &filename;
+		return true;
+	}
+
 	switch ( aviFormat )
 	{
 
@@ -668,9 +666,21 @@ bool AVIHandler::Create( const string& filename )
 }
 
 
-int AVIHandler::Write( Frame& frame )
+int AVIHandler::Write( Frame *frame )
 {
+	if ( !infoSet )
+	{
+		assert( !frame->IsHDV() );
+		SetSampleFrame( (DVFrame*)frame );
+		if ( ! Create( *filen ) )
+		{
+			sendEvent( ">>> Error creating file!" );
+			return false;
+		}
+	}
+
 	assert( avi != NULL );
+
 	return ( avi->WriteFrame( frame ) ? 0 : -1 );
 }
 
@@ -724,10 +734,10 @@ bool AVIHandler::Open( const char *s )
 
 }
 
-int AVIHandler::GetFrame( Frame &frame, int frameNum )
+
+int AVIHandler::GetFrame( Frame *frame, int frameNum )
 {
-	int result = avi->GetDVFrame( frame, frameNum );
-	frame.ExtractHeader();
+	int result = avi->GetFrame( frame, frameNum );
 	return result;
 }
 
@@ -738,7 +748,7 @@ void AVIHandler::SetOpenDML( bool flag )
 }
 
 
-bool AVIHandler::GetOpenDML() const
+bool AVIHandler::GetOpenDML()
 {
 	return isOpenDML;
 }
@@ -788,10 +798,9 @@ bool QtHandler::FileIsOpen()
 bool QtHandler::Create( const string& filename )
 {
 	Init();
-
 	fd = quicktime_open( const_cast<char*>( filename.c_str() ), 0, 1 );
+	this->filename = filename;
 
-	//if (fd != NULL) FileTracker::GetInstance().Add(filename.c_str());
 	return ( fd != NULL );
 }
 
@@ -810,13 +819,16 @@ inline void QtHandler::DeinterlaceStereo16( void* pInput, int iBytes,
 	}
 }
 
-int QtHandler::Write( Frame& frame )
+int QtHandler::Write( Frame *f )
 {
+	assert( !f->IsHDV() );
+	DVFrame *frame = (DVFrame*)f;
+
 	if ( ! isFullyInitialized )
 	{
 		AudioInfo audio;
 
-		if ( frame.GetAudioInfo( audio ) )
+		if ( frame->GetAudioInfo( audio ) )
 		{
 			/* TODO: handle 12-bit, non-linear audio */
 			channels = 2;
@@ -828,8 +840,8 @@ int QtHandler::Write( Frame& frame )
 			channels = 0;
 		}
 
-		quicktime_set_video( fd, 1, 720, frame.IsPAL() ? 576 : 480,
-		                     frame.GetFrameRate(), QUICKTIME_DV );
+		quicktime_set_video( fd, 1, 720, frame->IsPAL() ? 576 : 480,
+		                     frame->GetFrameRate(), QUICKTIME_DV );
 
 		if ( channels > 0 )
 		{
@@ -856,16 +868,16 @@ int QtHandler::Write( Frame& frame )
 		isFullyInitialized = true;
 	}
 
-	int result = quicktime_write_frame( fd, const_cast<unsigned char*>( frame.data ),
-	                                    frame.GetFrameSize(), 0 );
+	int result = quicktime_write_frame( fd, const_cast<unsigned char*>( frame->data ),
+	                                    frame->GetDataLen(), 0 );
 
 	if ( channels > 0 )
 	{
 		AudioInfo audio;
-		frame.ExtractHeader();
-		if ( frame.GetAudioInfo( audio ) && ( unsigned int ) audio.samples < audioBufferSize )
+		frame->ExtractHeader();
+		if ( frame->GetAudioInfo( audio ) && ( unsigned int ) audio.samples < audioBufferSize )
 		{
-			long bytesRead = frame.ExtractAudio( audioBuffer );
+			long bytesRead = frame->ExtractAudio( audioBuffer );
 
 			DeinterlaceStereo16( audioBuffer, bytesRead,
 			                     audioChannelBuffer[ 0 ],
@@ -907,7 +919,7 @@ off_t QtHandler::GetFileSize()
 	if ( fd )
 	{
 		struct stat file_status;
-		fstat( fileno( fd->stream ), &file_status );
+		stat( filename.c_str(), &file_status );
 		return file_status.st_size;
 	}
 	else
@@ -945,16 +957,16 @@ bool QtHandler::Open( const char *s )
 		Close();
 		return false;
 	}
+	filename = s;
 
 	return true;
 }
 
-int QtHandler::GetFrame( Frame &frame, int frameNum )
+int QtHandler::GetFrame( Frame *frame, int frameNum )
 {
 	assert( fd != NULL );
 	quicktime_set_video_position( fd, frameNum, 0 );
-	frame.bytesInFrame = quicktime_read_frame( fd, frame.data, 0 );
-	frame.ExtractHeader();
+	frame->SetDataLen( quicktime_read_frame( fd, frame->data, 0 ) );
 	return 0;
 }
 #endif
@@ -964,7 +976,7 @@ int QtHandler::GetFrame( Frame &frame, int frameNum )
 #if defined(HAVE_LIBJPEG) && defined(HAVE_LIBDV)
 
 JPEGHandler::JPEGHandler( int quality, bool deinterlace, int width, int height,
-                          bool overwrite ) :
+                          bool overwrite, string temp, bool usetemp ) :
 		isOpen( false ), count( 0 ), deinterlace( deinterlace ), overwrite( overwrite )
 {
 	extension = ".jpg";
@@ -976,6 +988,8 @@ JPEGHandler::JPEGHandler( int quality, bool deinterlace, int width, int height,
 	jpeg_set_quality( &cinfo, quality, TRUE /* limit to baseline-JPEG values */ );
 	new_height = CLAMP( height, -1, 2048 );
 	new_width = CLAMP( width, -1, 2048 );
+	this->temp=temp;
+	this->usetemp=usetemp;
 }
 
 
@@ -995,13 +1009,15 @@ bool JPEGHandler::Create( const string& filename )
 
 /* this must be called before scaling */
 /* height is fixed, returns new width */
-int JPEGHandler::fixAspect( Frame &frame )
+int JPEGHandler::fixAspect( Frame *frame )
 {
-	int width = frame.GetWidth( );
-	int height = frame.GetHeight( );
+	assert( !frame->IsHDV() );
+	DVFrame *dvframe = (DVFrame*)frame;
+	int width = frame->GetWidth( );
+	int height = frame->GetHeight( );
 	static JSAMPLE image[ 2048 * 2048 * 3 ];
 	register JSAMPLE *dest = image_buffer, *src = image;
-	int new_width = frame.IsPAL() ? 337 : 320;
+	int new_width = dvframe->IsPAL() ? 337 : 320;
 	int n = width / 2 - new_width;
 	int d = width / 2;
 	int a = n;
@@ -1030,10 +1046,10 @@ int JPEGHandler::fixAspect( Frame &frame )
 	return new_width;
 }
 
-bool JPEGHandler::scale( Frame &frame )
+bool JPEGHandler::scale( Frame *frame )
 {
-	int width = frame.GetWidth( );
-	int height = frame.GetHeight( );
+	int width = frame->GetWidth( );
+	int height = frame->GetHeight( );
 	static JSAMPLE image[ 2048 * 2048 * 3 ];
 	register JSAMPLE *dest = image_buffer, *src = image;
 	AffineTransform affine;
@@ -1092,20 +1108,21 @@ bool JPEGHandler::scale( Frame &frame )
 }
 
 
-int JPEGHandler::Write( Frame &frame )
+int JPEGHandler::Write( Frame *frame )
 {
+	assert( !frame->IsHDV() );
+	DVFrame *dvframe = (DVFrame*)frame;
 	JSAMPROW row_pointer[ 1 ];	/* pointer to JSAMPLE row[s] */
 	int row_stride;		/* physical row width in image buffer */
-	JDIMENSION width = frame.GetWidth();
-	JDIMENSION height = frame.GetHeight();
-	string file;
+	JDIMENSION width = frame->GetWidth();
+	JDIMENSION height = frame->GetHeight();
 
-	frame.ExtractHeader();
-	if ( frame.IsNewRecording() && GetAutoSplit() )
+	dvframe->ExtractHeader();
+	if ( frame->IsNewRecording() && GetAutoSplit() )
 		count = 0;
-	frame.ExtractRGB( image_buffer );
+	dvframe->ExtractRGB( image_buffer );
 	if ( deinterlace )
-		frame.Deinterlace( image_buffer, 3 );
+		dvframe->Deinterlace( image_buffer, 3 );
 	if ( new_width != -1 || new_height != -1 )
 	{
 		if ( new_width == -1 )
@@ -1128,9 +1145,16 @@ int JPEGHandler::Write( Frame &frame )
 
 	if ( overwrite )
 	{
-		ostringstream sb;
-		sb << GetBaseName() << GetExtension();
-		file = sb.str();
+		if ( StringUtils::ends( GetBaseName(), ".jpg" ) || StringUtils::ends( GetBaseName(), ".jpeg" ) )
+		{
+			file = GetBaseName();
+		}
+		else
+		{
+			ostringstream sb;
+			sb << GetBaseName() << GetExtension();
+			file = sb.str();
+		}
 	}
 	else
 	{
@@ -1138,7 +1162,14 @@ int JPEGHandler::Write( Frame &frame )
 		sb << filename.substr( 0, filename.find_last_of( '.' ) ) << "-" << setfill( '0' ) << setw( 8 ) << ++count << GetExtension();
 		file = sb.str();
 	}
-	FILE *outfile = fopen( const_cast<char*>( file.c_str() ), "wb" );
+
+	FILE *outfile;
+
+	if ( this->usetemp ) {
+		outfile = fopen( const_cast<char*>( this->temp.c_str() ), "wb" );
+	} else {
+		outfile = fopen( const_cast<char*>( file.c_str() ), "wb" );
+	}
 	if ( outfile != NULL )
 		jpeg_stdio_dest( &cinfo, outfile );
 	cinfo.image_width = new_width;
@@ -1152,6 +1183,9 @@ int JPEGHandler::Write( Frame &frame )
 	}
 	jpeg_finish_compress( &cinfo );
 	fclose( outfile );
+	if ( this->usetemp ) {
+		rename(const_cast<char*>( this->temp.c_str() ), const_cast<char*>( file.c_str() ));
+	}
 	return 0;
 }
 
@@ -1162,3 +1196,112 @@ int JPEGHandler::Close( void )
 }
 
 #endif
+
+
+/***************************************************************************/
+
+
+Mpeg2Handler::Mpeg2Handler( const string& ext ) :
+	fd( -1 ), waitingForRecordingDate( true ), bufferLen( 0 ), totalFrames( 0 )
+{
+	extension = ext;
+}
+
+Mpeg2Handler::~Mpeg2Handler()
+{
+	Close();
+}
+
+bool Mpeg2Handler::FileIsOpen()
+{
+	return fd != -1;
+}
+
+bool Mpeg2Handler::Create( const string& filename )
+{
+	if ( GetBaseName() == "-" )
+		fd = fileno( stdout );
+	else
+		fd = open( filename.c_str(), O_CREAT | O_TRUNC | O_RDWR | O_NONBLOCK, 0644 );
+	if ( fd != -1 )
+	{
+		FileTracker::GetInstance().Add( filename.c_str() );
+		this->filename = filename;
+	}
+	return ( fd != -1 );
+}
+
+bool Mpeg2Handler::WriteFrame( Frame *frame )
+{
+	if ( waitingForRecordingDate )
+	{
+		struct tm rd;
+		if ( !frame->GetRecordingDate( rd ) )
+		{
+			if ( bufferLen + frame->GetDataLen() < MPEG2_BUFFER_SIZE )
+			{
+				// Buffer up the first several frames until we get the recording date
+				memcpy( &buffer[bufferLen], frame->data, frame->GetDataLen() );
+				bufferLen += frame->GetDataLen();
+				totalFrames++;
+				return true;
+			}
+		}
+
+		waitingForRecordingDate = false;
+	}
+
+	return FileHandler::WriteFrame( frame );
+}
+
+int Mpeg2Handler::Write( Frame *frame )
+{
+	int result;
+
+	// Write any buffered data first.
+	if ( bufferLen > 0 )
+	{
+		if ( 0 > ( result = write( fd, buffer, bufferLen ) ) )
+			return result;
+		bufferLen = 0;
+	}
+
+	result = write( fd, frame->data, frame->GetDataLen() );
+
+	if ( 0 <= result )
+		totalFrames++;
+
+	return result;
+}
+
+int Mpeg2Handler::Close()
+{
+	if ( fd != -1 && fd != fileno( stdin ) && fd != fileno( stdout ) )
+	{
+		close( fd );
+		fd = -1;
+	}
+	return 0;
+}
+
+off_t Mpeg2Handler::GetFileSize()
+{
+	struct stat file_status;
+	fstat( fd, &file_status );
+	return file_status.st_size;
+}
+
+int Mpeg2Handler::GetTotalFrames()
+{
+	return totalFrames;
+}
+
+bool Mpeg2Handler::Open( const char *s )
+{
+	return false;
+}
+
+int Mpeg2Handler::GetFrame( Frame *frame, int frameNum )
+{
+	return -1;
+}
