@@ -22,7 +22,7 @@
 #ifndef _FILEHANDLER_H
 #define _FILEHANDLER_H
 
-enum { PAL_FORMAT, NTSC_FORMAT, AVI_DV1_FORMAT, AVI_DV2_FORMAT, QT_FORMAT, RAW_FORMAT, DIF_FORMAT, JPEG_FORMAT, UNDEFINED };
+enum { PAL_FORMAT, NTSC_FORMAT, AVI_DV1_FORMAT, AVI_DV2_FORMAT, QT_FORMAT, RAW_FORMAT, DIF_FORMAT, JPEG_FORMAT, MPEG2TS_FORMAT, UNDEFINED };
 
 #include <vector>
 using std::vector;
@@ -30,7 +30,8 @@ using std::vector;
 #include <string>
 using std::string;
 
-#include "frame.h"
+#include "dvframe.h"
+#include "hdvframe.h"
 #include "riff.h"
 #include "avi.h"
 #include <sys/types.h>
@@ -68,18 +69,18 @@ public:
 	FileHandler();
 	virtual ~FileHandler();
 
-	virtual bool GetAutoSplit() const;
-	virtual bool GetTimeStamp() const;
-	virtual bool GetTimeSys() const;
-	virtual bool GetTimeCode() const;
-	virtual string GetBaseName() const;
-	virtual string GetExtension() const;
-	virtual int GetMaxFrameCount() const;
-	virtual off_t GetMaxFileSize() const;
+	virtual bool GetAutoSplit();
+	virtual bool GetTimeStamp();
+	virtual bool GetTimeSys();
+	virtual bool GetTimeCode();
+	virtual string GetBaseName();
+	virtual string GetExtension();
+	virtual int GetMaxFrameCount();
+	virtual off_t GetMaxFileSize();
 	void CollectionCounterUpdate();
-	virtual int GetSizeSplitMode() const;
-	virtual off_t GetMinColSize() const;
-	virtual off_t GetMaxColSize() const;
+	virtual int GetSizeSplitMode();
+	virtual off_t GetMinColSize();
+	virtual off_t GetMaxColSize();
 	virtual off_t GetFileSize() = 0;
 	virtual int GetTotalFrames() = 0;
 
@@ -91,39 +92,47 @@ public:
 	virtual void SetMaxFrameCount( int );
 	virtual void SetEveryNthFrame( int );
 	virtual void SetMaxFileSize( off_t );
-	virtual void SetSampleFrame( const Frame& sample );
 	virtual void SetSizeSplitMode( int );
 	virtual void SetMinColSize( off_t );
 	virtual void SetMaxColSize( off_t );
 
-
-	virtual bool WriteFrame( Frame& frame );
+	virtual bool WriteFrame( Frame *frame );
 	virtual bool FileIsOpen() = 0;
 	virtual bool Create( const string& filename ) = 0;
-	virtual int Write( Frame& frame ) = 0;
+	virtual int Write( Frame *frame ) = 0;
 	virtual int Close() = 0;
 	virtual bool Done( void );
 
 	virtual bool Open( const char *s ) = 0;
-	virtual int GetFrame( Frame &frame, int frameNum ) = 0;
-	off_t GetLastCollectionFreeSpace() const
+	virtual int GetFrame( Frame *frame, int frameNum ) = 0;
+	off_t GetLastCollectionFreeSpace()
 	{
 		return lastCollectionFreeSpace;
 	}
-	off_t GetCurrentCollectionSize() const
+	off_t GetCurrentCollectionSize()
 	{
 		return currentCollectionSize;
 	}
-	int GetFramesWritten() const
+	int GetFramesWritten()
 	{
 		return framesWritten;
 	}
-	string GetFileName() const
+	virtual string GetFileName()
 	{
 		return filename;
 	}
+	virtual bool IsNewFile()
+	{
+		return isNewFile;
+	}
+	virtual bool IsFirstFile()
+	{
+		return isFirstFile == 0 ? false : true;
+	}
 
 protected:
+	int isFirstFile;
+	bool isNewFile;
 	bool done;
 	bool autoSplit;
 	bool timeStamp;
@@ -142,6 +151,7 @@ protected:
 	string base;
 	string extension;
 	string filename;
+	TimeCode prevTimeCode;
 };
 
 
@@ -155,12 +165,12 @@ public:
 
 	bool FileIsOpen();
 	bool Create( const string& filename );
-	int Write( Frame& frame );
+	int Write( Frame *frame );
 	int Close();
 	off_t GetFileSize();
 	int GetTotalFrames();
 	bool Open( const char *s );
-	int GetFrame( Frame &frame, int frameNum );
+	int GetFrame( Frame *frame, int frameNum );
 private:
 	int numBlocks;
 };
@@ -172,21 +182,23 @@ public:
 	AVIHandler( int format = AVI_DV1_FORMAT );
 	~AVIHandler();
 
-	void SetSampleFrame( const Frame& sample );
+	void SetSampleFrame( DVFrame *sample );
 	bool FileIsOpen();
 	bool Create( const string& filename );
-	int Write( Frame& frame );
+	int Write( Frame *frame );
 	int Close();
 	off_t GetFileSize();
 	int GetTotalFrames();
 	bool Open( const char *s );
-	int GetFrame( Frame &frame, int frameNum );
-	bool GetOpenDML() const;
+	int GetFrame( Frame *frame, int frameNum );
+	bool GetOpenDML();
 	void SetOpenDML( bool );
 
 protected:
+	const string *filen;
 	AVIFile *avi;
 	int aviFormat;
+	bool infoSet;
 	AudioInfo audioInfo;
 	VideoInfo videoInfo;
 	bool	isOpenDML;
@@ -195,7 +207,7 @@ protected:
 };
 
 #ifdef HAVE_LIBQUICKTIME
-#include <quicktime/quicktime.h>
+#include <quicktime.h>
 
 class QtHandler: public FileHandler
 {
@@ -205,12 +217,12 @@ public:
 
 	bool FileIsOpen();
 	bool Create( const string& filename );
-	int Write( Frame& frame );
+	int Write( Frame *frame );
 	int Close();
 	off_t GetFileSize();
 	int GetTotalFrames();
 	bool Open( const char *s );
-	int GetFrame( Frame &frame, int frameNum );
+	int GetFrame( Frame *frame, int frameNum );
 
 private:
 	quicktime_t *fd;
@@ -251,12 +263,15 @@ private:
 	int new_width;
 	int new_height;
 	bool overwrite;
+	string file;
+	string temp;
+	bool usetemp;
 
-	int fixAspect( Frame &frame );
-	bool scale( Frame &frame );
+	int fixAspect( Frame *frame );
+	bool scale( Frame *frame );
 
 public:
-	JPEGHandler( int quality, bool deinterlace = false, int width = -1, int height = -1, bool overwrite = false );
+	JPEGHandler( int quality, bool deinterlace = false, int width = -1, int height = -1, bool overwrite = false, string temp = "tmp.jpg", bool usetemp = false );
 	~JPEGHandler();
 
 	bool FileIsOpen()
@@ -264,7 +279,7 @@ public:
 		return isOpen;
 	}
 	bool Create( const string& filename );
-	int Write( Frame& frame );
+	int Write( Frame *frame );
 	int Close();
 	off_t GetFileSize()
 	{
@@ -278,11 +293,42 @@ public:
 	{
 		return false;
 	}
-	int GetFrame( Frame &frame, int frameNum )
+	string GetFileName()
+	{
+		return file;
+	}
+	int GetFrame( Frame *frame, int frameNum )
 	{
 		return -1;
 	}
 };
 #endif
 
+class Mpeg2Handler: public FileHandler
+{
+public:
+	int fd;
+
+	Mpeg2Handler( const string& ext = string( ".m2t" ) );
+	~Mpeg2Handler();
+
+	bool WriteFrame( Frame *frame );
+	bool FileIsOpen();
+	bool Create( const string& filename );
+	int Write( Frame *frame );
+	int Close();
+	off_t GetFileSize();
+	int GetTotalFrames();
+	bool Open( const char *s );
+	int GetFrame( Frame *frame, int frameNum );
+
+private:
+#define MPEG2_BUFFER_SIZE (2*1024*1024)
+	bool waitingForRecordingDate;
+	unsigned char buffer[MPEG2_BUFFER_SIZE];
+	int bufferLen;
+
+	int totalFrames;
+};
+
 #endif
