@@ -50,7 +50,6 @@
 #include <deque>
 #include <iostream>
 
-using std::cerr;
 using std::endl;
 
 #include <assert.h>
@@ -69,7 +68,8 @@ using std::endl;
 #include <libavc1394/rom1394.h>
 
 #include "ieee1394io.h"
-#include "frame.h"
+#include "dvframe.h"
+#include "hdvframe.h"
 #include "error.h"
 
 /** Initializes the IEEE1394Reader object.
@@ -86,18 +86,23 @@ using std::endl;
  */
 
 
-IEEE1394Reader::IEEE1394Reader( int c, int bufSize ) :
+IEEE1394Reader::IEEE1394Reader( int c, int bufSize, bool hdv ) :
 	droppedFrames( 0 ),
-	incompleteFrames( 0 ),
+	currentFrame( NULL ),
 	channel( c ),
-	isRunning( false )
+	isRunning( false ),
+	isHDV( hdv )
 {
 	Frame * frame;
 
 	/* Create empty frames and put them in our inFrames queue */
 	for ( int i = 0; i < bufSize; ++i )
 	{
-		frame = new Frame();
+		if ( isHDV )
+			frame = new HDVFrame( &hdvStreamParams );
+		else
+			frame = new DVFrame();
+
 		inFrames.push_back( frame );
 	}
 
@@ -172,8 +177,6 @@ Frame* IEEE1394Reader::GetFrame()
 		outFrames.pop_front();
 	}
 	pthread_mutex_unlock( &mutex );
-	if ( frame != NULL )
-		frame->ExtractHeader();
 
 	return frame;
 }
@@ -203,17 +206,6 @@ int IEEE1394Reader::GetDroppedFrames( void )
 }
 
 
-/** Return the number of incomplete frames since last call
-*/
-
-int IEEE1394Reader::GetIncompleteFrames( void )
-{
-	int n = incompleteFrames;
-	incompleteFrames = 0;
-	return n;
-}
-
-
 /** Throw away all currently available frames.
  
     All frames in the outFrames queue are put back to the inFrames
@@ -298,8 +290,8 @@ void IEEE1394Reader::TriggerAction( )
 
 
 iec61883Reader::iec61883Reader( int p, int c, int bufSize,
-	BusResetHandler resetHandler, BusResetHandlerData data ) :
-		IEEE1394Reader( c, bufSize ), m_port( p ), m_resetHandler( resetHandler),
+	BusResetHandler resetHandler, BusResetHandlerData data, bool hdv ) :
+		IEEE1394Reader( c, bufSize, hdv ), m_port( p ), m_resetHandler( resetHandler),
 		m_resetHandlerData( data )
 {
 	m_handle = NULL;
@@ -374,11 +366,23 @@ void iec61883Reader::ResetHandler( void )
 
 int iec61883Reader::ResetHandlerProxy( raw1394handle_t handle, unsigned int generation )
 {
-	iec61883_dv_t dv = static_cast< iec61883_dv_t >( raw1394_get_userdata( handle ) );
-	iec61883_dv_fb_t dvfb = static_cast< iec61883_dv_fb_t >( iec61883_dv_get_callback_data( dv ) );
-	iec61883Reader *self = static_cast< iec61883Reader* >( iec61883_dv_fb_get_callback_data( dvfb ) );
+	iec61883Reader *self = NULL;
+	void *userdata = raw1394_get_userdata( handle );
+
+	if ( typeid( iec61883_mpeg2_t ) == typeid( userdata ) )
+	{
+		iec61883_mpeg2_t mpeg2 = static_cast< iec61883_mpeg2_t >( userdata );
+		self = static_cast< iec61883Reader* >( iec61883_mpeg2_get_callback_data( mpeg2 ) );
+	}
+	else if ( typeid( iec61883_dv_t ) == typeid( userdata ) )
+	{
+		iec61883_dv_t dv = static_cast< iec61883_dv_t >( userdata );
+		iec61883_dv_fb_t dvfb = static_cast< iec61883_dv_fb_t >( iec61883_dv_get_callback_data( dv ) );
+		self = static_cast< iec61883Reader* >( iec61883_dv_fb_get_callback_data( dvfb ) );
+	}
 	if ( self )
 		self->ResetHandler();
+
 	return 0;
 }
 
@@ -401,13 +405,21 @@ bool iec61883Reader::Open()
 			return false;
 		raw1394_set_bus_reset_handler( m_handle, this->ResetHandlerProxy );
 
-		m_iec61883dv = iec61883_dv_fb_init( m_handle, HandlerProxy, this );
-		success = ( m_iec61883dv != NULL );
+		if ( isHDV )
+		{
+			m_iec61883.mpeg2 = iec61883_mpeg2_recv_init( m_handle, Mpeg2HandlerProxy, this );
+			success = ( m_iec61883.mpeg2 != NULL );
+		}
+		else
+		{
+			m_iec61883.dv = iec61883_dv_fb_init( m_handle, DvHandlerProxy, this );
+			success = ( m_iec61883.dv != NULL );
+		}
 	}
 	catch ( string exc )
 	{
 		Close();
-		cerr << exc << endl;
+		sendEvent( exc.c_str() );
 		success = false;
 	}
 	return success;
@@ -420,10 +432,14 @@ bool iec61883Reader::Open()
 
 void iec61883Reader::Close()
 {
-	if ( m_iec61883dv != NULL )
+	if ( m_iec61883.ref != NULL )
 	{
-		iec61883_dv_fb_close( m_iec61883dv );
-		m_iec61883dv = NULL;
+		if ( isHDV )
+			iec61883_mpeg2_close( m_iec61883.mpeg2 );
+		else
+			iec61883_dv_fb_close( m_iec61883.dv );
+
+		m_iec61883.dv = NULL;
 	}
 }
 
@@ -434,12 +450,15 @@ bool iec61883Reader::StartReceive()
 	/* Starting iso receive */
 	try
 	{
-		fail_neg( iec61883_dv_fb_start( m_iec61883dv, channel ) );
+		if ( isHDV )
+			fail_neg( iec61883_mpeg2_recv_start( m_iec61883.mpeg2, channel ) );
+		else
+			fail_neg( iec61883_dv_fb_start( m_iec61883.dv, channel ) );
 		success = true;
 	}
 	catch ( string exc )
 	{
-		cerr << exc << endl;
+		sendEvent( exc.c_str() );
 		success = false;
 	}
 	return success;
@@ -448,53 +467,63 @@ bool iec61883Reader::StartReceive()
 
 void iec61883Reader::StopReceive()
 {
-	if ( m_iec61883dv != NULL )
+	if ( m_iec61883.ref != NULL )
 	{
-		iec61883_dv_fb_close( m_iec61883dv );
-		m_iec61883dv = NULL;
+		if ( isHDV )
+			iec61883_mpeg2_close( m_iec61883.mpeg2 );
+		else
+			iec61883_dv_fb_close( m_iec61883.dv );
+
+		m_iec61883.ref = NULL;
 	}
 }
 
-int iec61883Reader::HandlerProxy( unsigned char *data, int length, int complete, 
-	void *callback_data )
+int iec61883Reader::Mpeg2HandlerProxy( unsigned char *data, int length,
+	unsigned int dropped, void *callback_data )
 {
 	iec61883Reader *self = static_cast< iec61883Reader* >( callback_data );
-	return self->Handler( length, complete, data );
+	return self->Handler( data, length, dropped );
 }
 
-int iec61883Reader::Handler( int length, int complete, unsigned char *data )
+int iec61883Reader::DvHandlerProxy( unsigned char *data, int length,
+	int complete, void *callback_data )
 {
-	pthread_mutex_lock( &mutex );
-	if ( currentFrame != NULL )
+	iec61883Reader *self = static_cast< iec61883Reader* >( callback_data );
+	return self->Handler( data, length, !complete );
+}
+
+int iec61883Reader::Handler( unsigned char *data, int length, int dropped )
+{
+	droppedFrames += dropped;
+
+	if ( currentFrame == NULL )
 	{
+		if ( inFrames.size() > 0 )
+		{
+			pthread_mutex_lock( &mutex );
+			currentFrame = inFrames.front();
+			currentFrame->Clear();
+			inFrames.pop_front();
+			pthread_mutex_unlock( &mutex );
+		}
+		else
+		{
+			droppedFrames++;
+			return 0;
+		}
+	}
+
+	memcpy( &currentFrame->data[currentFrame->GetDataLen()], data, length );
+	currentFrame->AddDataLen( length );
+
+	if ( currentFrame->IsComplete( ) )
+	{
+		pthread_mutex_lock( &mutex );
 		outFrames.push_back( currentFrame );
 		currentFrame = NULL;
-		//cerr << "reader > # inFrames: " << inFrames.size() << ", # outFrames: " << outFrames.size() << endl;
 		TriggerAction( );
+		pthread_mutex_unlock( &mutex );
 	}
-	if ( inFrames.size() > 0 )
-	{
-		currentFrame = inFrames.front();
-		currentFrame->bytesInFrame = 0;
-		inFrames.pop_front();
-		//if ( outFrames.size( ) >= 25 )
-		//cerr << "reader < # inFrames: " << inFrames.size() << ", # outFrames: " << outFrames.size() << endl;
-	}
-	else
-	{
-		droppedFrames++;
-		//cerr << "Warning: raw1394 dropped frames: " << droppedFrames << endl;
-	}
-	pthread_mutex_unlock( &mutex );
-
-	if ( currentFrame != NULL )
-	{
-		memcpy( currentFrame->data, data, length );
-		currentFrame->bytesInFrame = currentFrame->GetFrameSize( );
-	}
-
-	if ( !complete )
-		incompleteFrames++;
 
 	return 0;
 }
@@ -607,7 +636,7 @@ AVC::AVC( int p ) : port( p )
 		if ( avc_handle != NULL )
 			raw1394_destroy_handle( avc_handle );
 		avc_handle = NULL;
-		cerr << exc << endl;
+		sendEvent( exc.c_str() );
 	}
 	return ;
 }
@@ -976,7 +1005,7 @@ bool pipeReader::Handler()
 	if ( currentFrame == NULL && inFrames.size() > 0 )
 	{
 		currentFrame = inFrames.front();
-		currentFrame->bytesInFrame = 0;
+		currentFrame->Clear();
 		inFrames.pop_front();
 		//printf("reader < buf: buffer %d, output %d\n", inFrames.size(), outFrames.size());
 		//fflush(stdout);
@@ -984,26 +1013,33 @@ bool pipeReader::Handler()
 	pthread_mutex_unlock( &mutex );
 	if ( currentFrame != NULL )
 	{
-		ret = ( fread( currentFrame->data, 120000, 1, stdin ) == 1 );
-		if ( ret && currentFrame->data[ 3 ] & 0x80 )
+		if ( isHDV )
 		{
-			ret = ( fread( currentFrame->data + 120000, 24000, 1, stdin ) == 1 );
-			if ( ret )
-				currentFrame->bytesInFrame = 144000;
+			void *buf = &currentFrame->data[currentFrame->GetDataLen()];
+			if ( ret = ( fread( buf, IEC61883_MPEG2_TSP_SIZE, 1, file ) == 1 ) )
+				currentFrame->AddDataLen( IEC61883_MPEG2_TSP_SIZE );
+			else
+				((HDVFrame*)currentFrame)->SetComplete();
 		}
-		else if ( ret )
+		else
 		{
-			currentFrame->bytesInFrame = 120000;
+			if ( ret = ( fread( currentFrame->data, 120000, 1, file ) == 1 ) )
+			{
+				currentFrame->SetDataLen( 120000 );
+
+				if ( currentFrame->data[ 3 ] & 0x80 )
+					if ( ret = ( fread( currentFrame->data + 120000, 24000, 1, file ) == 1 ) )
+						currentFrame->AddDataLen( 24000 );
+			}
 		}
-		if ( ret )
+
+		if ( ( ret && currentFrame->IsComplete() ) || ( !ret && currentFrame->GetDataLen() > 0 ) )
 		{
 			pthread_mutex_lock( &mutex );
 			outFrames.push_back( currentFrame );
 			currentFrame = NULL;
 			TriggerAction( );
 			pthread_mutex_unlock( &mutex );
-			//printf("reader > out: buffer %d, output %d\n", inFrames.size(), outFrames.size());
-			//fflush(stdout);
 		}
 	}
 	return ret;
@@ -1024,16 +1060,31 @@ void* pipeReader::ThreadProxy( void* arg )
 
 void* pipeReader::Thread()
 {
+	if ( strcmp( input_file, "-" ) == 0 )
+		file = stdin;
+	else
+		file = fopen( input_file, "rb" );
+
+	if ( ! file )
+	{
+		sendEvent( "No input file" );
+		return NULL;
+	}
+
 	while ( true )
 	{
 		if ( ! Handler() )
 			break;
 		pthread_testcancel();
 	}
-	cerr << "End of pipe\n";
+
+	if ( strcmp( input_file, "-" ) != 0 )
+		fclose( file );
+
+	sendEvent( "End of pipe" );
 	pthread_mutex_lock( &mutex );
+	if ( currentFrame ) outFrames.push_back( currentFrame );
 	currentFrame = NULL;
-	outFrames.push_back( currentFrame );
 	TriggerAction( );
 	pthread_mutex_unlock( &mutex );
 	return NULL;
