@@ -30,6 +30,8 @@ using std::string;
 #include <deque>
 using std::deque;
 
+#include "hdvframe.h"
+
 class Frame;
 
 class IEEE1394Reader
@@ -39,10 +41,6 @@ protected:
 	/// our inFrames queue did not contain available frames
 	int droppedFrames;
 
-	/// the number of frames that are tainted because they are incomplete -
-	/// a packet was dropped
-	int incompleteFrames;
-
 	/// a pointer to the frame which is currently been transmitted
 	Frame	*currentFrame;
 
@@ -54,7 +52,7 @@ protected:
 
 public:
 
-	IEEE1394Reader( int channel = 63, int frames = 50 );
+	IEEE1394Reader( int channel = 63, int frames = 50, bool hdv = false );
 	virtual ~IEEE1394Reader();
 
 	// Mutex protected public methods
@@ -63,7 +61,6 @@ public:
 	Frame* GetFrame( void );
 	void DoneWithFrame( Frame* );
 	int GetDroppedFrames( void );
-	int GetIncompleteFrames( void );
 	int GetOutQueueSize( void )
 	{
 		return outFrames.size();
@@ -102,6 +99,10 @@ protected:
 	/// A state variable for starting and stopping thread
 	bool isRunning;
 
+	/// If this is handling DV or HDV
+	bool isHDV;
+	HDVStreamParams hdvStreamParams;
+
 	void Flush( void );
 };
 
@@ -120,7 +121,11 @@ private:
 	raw1394handle_t m_handle;
 
 	/// the handle to libiec61883
-	iec61883_dv_fb_t m_iec61883dv;
+	union {
+		void *ref;
+		iec61883_dv_fb_t dv;
+		iec61883_mpeg2_t mpeg2;
+	} m_iec61883;
 
 	BusResetHandler m_resetHandler;
 	const void* m_resetHandlerData;
@@ -128,7 +133,7 @@ private:
 public:
 
 	iec61883Reader( int port = 0, int channel = 63, int buffers = 50, 
-		BusResetHandler = 0, BusResetHandlerData = 0 );
+		BusResetHandler = 0, BusResetHandlerData = 0, bool hdv = false );
 	~iec61883Reader();
 
 	bool Open( void );
@@ -137,13 +142,15 @@ public:
 	void StopReceive( void );
 	bool StartThread( void );
 	void StopThread( void );
-	int Handler( int length, int complete, unsigned char *data );
+	int Handler( unsigned char *data, int length, int dropped );
 	void *Thread();
 	void ResetHandler( void );
 
 private:
 	static int ResetHandlerProxy( raw1394handle_t handle, unsigned int generation );
-	static int HandlerProxy( unsigned char *data, int length, int complete, 
+	static int Mpeg2HandlerProxy( unsigned char *data, int length, unsigned int dropped, 
+		void *callback_data );
+	static int DvHandlerProxy( unsigned char *data, int length, int complete, 
 		void *callback_data );
 	static void* ThreadProxy( void *arg );
 };
@@ -217,8 +224,8 @@ class pipeReader: public IEEE1394Reader
 {
 public:
 
-	pipeReader( int frames = 50 ) :
-			IEEE1394Reader( 0, frames )
+	pipeReader( char *filename, int frames = 50, bool hdv = false ) :
+		IEEE1394Reader( 0, frames, hdv ), input_file( filename )
 	{};
 	~pipeReader()
 	{};
@@ -243,6 +250,9 @@ public:
 private:
 	bool Handler();
 	static void* ThreadProxy( void *arg );
+
+	FILE *file;
+	char *input_file;
 };
 
 
